package com.gyee.frame.controller.gen;

import java.util.List;

import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;
import com.github.pagehelper.PageInfo;
import com.gyee.frame.common.base.BaseController;
import com.gyee.frame.common.domain.AjaxResult;
import com.gyee.frame.model.auto.Gyeepermission;
import com.gyee.frame.model.auto.GyeepermissionView;
import com.gyee.frame.model.auto.Gyeerole;
import com.gyee.frame.model.custom.TableSplitResult;
import com.gyee.frame.model.custom.Tablepar;
import com.gyee.frame.model.custom.TitleVo;
import com.gyee.frame.service.GyeepermissionService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

@Controller
@RequestMapping("GyeepermissionController")
@Api(value = "")
public class GyeepermissionController extends BaseController{
	
	private String prefix = "gen/gyeepermission";
	@Autowired
	private GyeepermissionService gyeepermissionService;
	
	@GetMapping("view")
	@RequiresPermissions("gen:gyeepermission:view")
    public String view(ModelMap model)
    {	
		String str="";
		setTitle(model, new TitleVo("列表", str+"管理", true,"欢迎进入"+str+"页面", true, false));
        return prefix + "/list";
    }
	
	/**
	 * 
		 *@author:Li Miao
		 *
		 *@description: 得到模块权限
		 *
		 *@param:roleId
		 *
		 *@return:
		 *
		 *@createTime:2020年4月20日
		 *
	 */
	@GetMapping("/GyeePermissionList")
	@ResponseBody
	public AjaxResult getPermission(String userid){
		List<GyeepermissionView> list=gyeepermissionService.queryPermissionByRoleId(userid);
		if(list.isEmpty()){
			return AjaxResult.error(500, "无法获取权限");
			
		}else{
			return AjaxResult.successData(200, list);
		}
	}
	
	
	//@Log(title = "集合查询", action = "111")
	@PostMapping("list")
	@RequiresPermissions("gen:gyeepermission:list")
	@ResponseBody
	public Object list(Tablepar tablepar,String searchText){
		PageInfo<Gyeepermission> page=gyeepermissionService.list(tablepar,searchText) ; 
		TableSplitResult<Gyeepermission> result=new TableSplitResult<Gyeepermission>(page.getPageNum(), page.getTotal(), page.getList()); 
		return  result;
	}
	
	/**
     * 新增
     */

    @GetMapping("/add")
    public String add(ModelMap modelMap)
    {
        return prefix + "/add";
    }
	
	//@Log(title = "新增", action = "111")
	@PostMapping("add")
	@RequiresPermissions("gen:gyeepermission:add")
	@ResponseBody
	public AjaxResult add(Gyeepermission gyeepermission){
		int b=gyeepermissionService.insertSelective(gyeepermission);
		if(b>0){
			return success();
		}else{
			return error();
		}
	}
	
	/**
	 * 删除用户
	 * @param ids
	 * @return
	 */
	//@Log(title = "删除", action = "111")
	@PostMapping("remove")
	@RequiresPermissions("gen:gyeepermission:remove")
	@ResponseBody
	public AjaxResult remove(String ids){
		int b=gyeepermissionService.deleteByPrimaryKey(ids);
		if(b>0){
			return success();
		}else{
			return error();
		}
	}
	
	/**
	 * 检查用户
	 * @param tsysUser
	 * @return
	 */
	@PostMapping("checkNameUnique")
	@ResponseBody
	public int checkNameUnique(Gyeepermission gyeepermission){
		int b=gyeepermissionService.checkNameUnique(gyeepermission);
		if(b>0){
			return 1;
		}else{
			return 0;
		}
	}
	
	
	/**
	 * 修改跳转
	 * @param id
	 * @param mmap
	 * @return
	 */
	@GetMapping("/edit/{id}")
    public String edit(@PathVariable("id") String id, ModelMap mmap)
    {
        mmap.put("Gyeepermission", gyeepermissionService.selectByPrimaryKey(id));

        return prefix + "/edit";
    }
	
	/**
     * 修改保存
     */
    //@Log(title = "修改", action = "111")
    @RequiresPermissions("gen:gyeepermission:edit")
    @PostMapping("/edit")
    @ResponseBody
    public AjaxResult editSave(Gyeepermission record)
    {
        return toAjax(gyeepermissionService.updateByPrimaryKeySelective(record));
    }

    
    /**
   	 * 根据主键查询
   	 * 
   	 * @param id
   	 * @param mmap
   	 * @return
   	 */
   	@PostMapping("/get/{id}")
   	@ApiOperation(value = "根据id查询唯一", notes = "根据id查询唯一")
   	public Gyeepermission edit(@PathVariable("id") String id) {
   		return gyeepermissionService.selectByPrimaryKey(id);
   	}
    

	
}
