package com.gyee.frame.controller.gen;

import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;
import com.github.pagehelper.PageInfo;
import com.gyee.frame.common.base.BaseController;
import com.gyee.frame.common.domain.AjaxResult;
import com.gyee.frame.model.auto.Gyeeuser;
import com.gyee.frame.model.custom.TableSplitResult;
import com.gyee.frame.model.custom.Tablepar;
import com.gyee.frame.model.custom.TitleVo;
import com.gyee.frame.netty.websocket.util.SocketToolController;
import com.gyee.frame.service.GyeeuserService;

import io.lettuce.core.dynamic.annotation.Param;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

@Controller
@RequestMapping("GyeeuserController")
@Api(value = "")
public class GyeeuserController extends SocketToolController{
	
	private String prefix = "gen/gyeeuser";
	@Autowired
	private GyeeuserService gyeeuserService;
	
	@GetMapping("view")
	@RequiresPermissions("gen:gyeeuser:view")
    public String view(ModelMap model)
    {	
		String str="";
		setTitle(model, new TitleVo("列表", str+"管理", true,"欢迎进入"+str+"页面", true, false));
        return prefix + "/list";
    }
	
	/**
	 * 
		 *@author:Li Miao
		 *
		 *@description: 登录接口
		 *
		 *@param:username,password
		 *
		 *@return:
		 *
		 *@createTime:2020年4月16日
		 *
	 */
	@GetMapping("/login")
	@ResponseBody
	public AjaxResult login(String username,String password){
		Gyeeuser gyeeuser=new Gyeeuser();
		gyeeuser.setUsername(username);
		gyeeuser.setPassword(password);
		String id=gyeeuserService.queryIdByUserNamePassWord(gyeeuser);
		if(id!=""&&id!=null){
			return AjaxResult.successData(200, id);
			
		}else{			
			return AjaxResult.successData(200, "-1");
		}
	}
	
	
	//@Log(title = "集合查询", action = "111")
	@PostMapping("list")
	@RequiresPermissions("gen:gyeeuser:list")
	@ResponseBody
	public Object list(Tablepar tablepar,String searchText){
		PageInfo<Gyeeuser> page=gyeeuserService.list(tablepar,searchText) ; 
		TableSplitResult<Gyeeuser> result=new TableSplitResult<Gyeeuser>(page.getPageNum(), page.getTotal(), page.getList()); 
		return  result;
	}
	
	/**
     * 新增
     */

    @GetMapping("/add")
    public String add(ModelMap modelMap)
    {
        return prefix + "/add";
    }
	
	//@Log(title = "新增", action = "111")
	@PostMapping("add")
	@RequiresPermissions("gen:gyeeuser:add")
	@ResponseBody
	public AjaxResult add(Gyeeuser gyeeuser){
		int b=gyeeuserService.insertSelective(gyeeuser);
		if(b>0){
			return success();
		}else{
			return error();
		}
	}
	
	/**
	 * 删除用户
	 * @param ids
	 * @return
	 */
	//@Log(title = "删除", action = "111")
	@PostMapping("remove")
	@RequiresPermissions("gen:gyeeuser:remove")
	@ResponseBody
	public AjaxResult remove(String ids){
		int b=gyeeuserService.deleteByPrimaryKey(ids);
		if(b>0){
			return success();
		}else{
			return error();
		}
	}
	
	/**
	 * 检查用户
	 * @param tsysUser
	 * @return
	 */
	@PostMapping("checkNameUnique")
	@ResponseBody
	public int checkNameUnique(Gyeeuser gyeeuser){
		int b=gyeeuserService.checkNameUnique(gyeeuser);
		if(b>0){
			return 1;
		}else{
			return 0;
		}
	}
	
	
	/**
	 * 修改跳转
	 * @param id
	 * @param mmap
	 * @return
	 */
	@GetMapping("/edit/{id}")
    public String edit(@PathVariable("id") String id, ModelMap mmap)
    {
        mmap.put("Gyeeuser", gyeeuserService.selectByPrimaryKey(id));

        return prefix + "/edit";
    }
	
	/**
     * 修改保存
     */
    //@Log(title = "修改", action = "111")
    @RequiresPermissions("gen:gyeeuser:edit")
    @PostMapping("/edit")
    @ResponseBody
    public AjaxResult editSave(Gyeeuser record)
    {
        return toAjax(gyeeuserService.updateByPrimaryKeySelective(record));
    }

    
    /**
   	 * 根据主键查询
   	 * 
   	 * @param id
   	 * @param mmap
   	 * @return
   	 */
   	@PostMapping("/get/{id}")
   	@ApiOperation(value = "根据id查询唯一", notes = "根据id查询唯一")
   	public Gyeeuser edit(@PathVariable("id") String id) {
   		return gyeeuserService.selectByPrimaryKey(id);
   	}
    

	
}
