package com.gyee.frame.controller.gen;

import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;
import com.github.pagehelper.PageInfo;
import com.gyee.frame.common.base.BaseController;
import com.gyee.frame.common.domain.AjaxResult;
import com.gyee.frame.model.auto.Line;
import com.gyee.frame.model.custom.TableSplitResult;
import com.gyee.frame.model.custom.Tablepar;
import com.gyee.frame.model.custom.TitleVo;
import com.gyee.frame.service.LineService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

@Controller
@RequestMapping("LineController")
@Api(value = "线路")
public class LineController extends BaseController{
	
	private String prefix = "gen/line";
	@Autowired
	private LineService lineService;
	
	@GetMapping("view")
	@RequiresPermissions("gen:line:view")
    public String view(ModelMap model)
    {	
		String str="线路";
		setTitle(model, new TitleVo("列表", str+"管理", true,"欢迎进入"+str+"页面", true, false));
        return prefix + "/list";
    }
	
	//@Log(title = "线路集合查询", action = "111")
	@PostMapping("list")
	@RequiresPermissions("gen:line:list")
	@ResponseBody
	public Object list(Tablepar tablepar,String searchText){
		PageInfo<Line> page=lineService.list(tablepar,searchText) ; 
		TableSplitResult<Line> result=new TableSplitResult<Line>(page.getPageNum(), page.getTotal(), page.getList()); 
		return  result;
	}
	
	/**
     * 新增
     */

    @GetMapping("/add")
    public String add(ModelMap modelMap)
    {
        return prefix + "/add";
    }
	
	//@Log(title = "线路新增", action = "111")
	@PostMapping("add")
	@RequiresPermissions("gen:line:add")
	@ResponseBody
	public AjaxResult add(Line line){
		int b=lineService.insertSelective(line);
		if(b>0){
			return success();
		}else{
			return error();
		}
	}
	
	/**
	 * 删除用户
	 * @param ids
	 * @return
	 */
	//@Log(title = "线路删除", action = "111")
	@PostMapping("remove")
	@RequiresPermissions("gen:line:remove")
	@ResponseBody
	public AjaxResult remove(String ids){
		int b=lineService.deleteByPrimaryKey(ids);
		if(b>0){
			return success();
		}else{
			return error();
		}
	}
	
	/**
	 * 检查用户
	 * @param tsysUser
	 * @return
	 */
	@PostMapping("checkNameUnique")
	@ResponseBody
	public int checkNameUnique(Line line){
		int b=lineService.checkNameUnique(line);
		if(b>0){
			return 1;
		}else{
			return 0;
		}
	}
	
	
	/**
	 * 修改跳转
	 * @param id
	 * @param mmap
	 * @return
	 */
	@GetMapping("/edit/{id}")
    public String edit(@PathVariable("id") String id, ModelMap mmap)
    {
        mmap.put("Line", lineService.selectByPrimaryKey(id));

        return prefix + "/edit";
    }
	
	/**
     * 修改保存
     */
    //@Log(title = "线路修改", action = "111")
    @RequiresPermissions("gen:line:edit")
    @PostMapping("/edit")
    @ResponseBody
    public AjaxResult editSave(Line record)
    {
        return toAjax(lineService.updateByPrimaryKeySelective(record));
    }

    
    /**
   	 * 根据主键查询
   	 * 
   	 * @param id
   	 * @param mmap
   	 * @return
   	 */
   	@PostMapping("/get/{id}")
   	@ApiOperation(value = "根据id查询唯一", notes = "根据id查询唯一")
   	public Line edit(@PathVariable("id") String id) {
   		return lineService.selectByPrimaryKey(id);
   	}
    

	
}
