package com.gyee.frame.controller.gen;

import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;
import com.github.pagehelper.PageInfo;
import com.gyee.frame.common.base.BaseController;
import com.gyee.frame.common.domain.AjaxResult;
import com.gyee.frame.model.auto.Windturbine;
import com.gyee.frame.model.custom.TableSplitResult;
import com.gyee.frame.model.custom.Tablepar;
import com.gyee.frame.model.custom.TitleVo;
import com.gyee.frame.service.WindturbineService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

@Controller
@RequestMapping("WindturbineController")
@Api(value = "风机信息表")
public class WindturbineController extends BaseController{
	
	private String prefix = "gen/windturbine";
	@Autowired
	private WindturbineService windturbineService;
	
	@GetMapping("view")
	@RequiresPermissions("gen:windturbine:view")
    public String view(ModelMap model)
    {	
		String str="风机信息表";
		setTitle(model, new TitleVo("列表", str+"管理", true,"欢迎进入"+str+"页面", true, false));
        return prefix + "/list";
    }
	
	//@Log(title = "风机信息表集合查询", action = "111")
	@PostMapping("list")
	@RequiresPermissions("gen:windturbine:list")
	@ResponseBody
	public Object list(Tablepar tablepar,String searchText){
		PageInfo<Windturbine> page=windturbineService.list(tablepar,searchText) ; 
		TableSplitResult<Windturbine> result=new TableSplitResult<Windturbine>(page.getPageNum(), page.getTotal(), page.getList()); 
		return  result;
	}
	
	/**
     * 新增
     */

    @GetMapping("/add")
    public String add(ModelMap modelMap)
    {
        return prefix + "/add";
    }
	
	//@Log(title = "风机信息表新增", action = "111")
	@PostMapping("add")
	@RequiresPermissions("gen:windturbine:add")
	@ResponseBody
	public AjaxResult add(Windturbine windturbine){
		int b=windturbineService.insertSelective(windturbine);
		if(b>0){
			return success();
		}else{
			return error();
		}
	}
	
	/**
	 * 删除用户
	 * @param ids
	 * @return
	 */
	//@Log(title = "风机信息表删除", action = "111")
	@PostMapping("remove")
	@RequiresPermissions("gen:windturbine:remove")
	@ResponseBody
	public AjaxResult remove(String ids){
		int b=windturbineService.deleteByPrimaryKey(ids);
		if(b>0){
			return success();
		}else{
			return error();
		}
	}
	
	/**
	 * 检查用户
	 * @param tsysUser
	 * @return
	 */
	@PostMapping("checkNameUnique")
	@ResponseBody
	public int checkNameUnique(Windturbine windturbine){
		int b=windturbineService.checkNameUnique(windturbine);
		if(b>0){
			return 1;
		}else{
			return 0;
		}
	}
	
	
	/**
	 * 修改跳转
	 * @param id
	 * @param mmap
	 * @return
	 */
	@GetMapping("/edit/{id}")
    public String edit(@PathVariable("id") String id, ModelMap mmap)
    {
        mmap.put("Windturbine", windturbineService.selectByPrimaryKey(id));

        return prefix + "/edit";
    }
	
	/**
     * 修改保存
     */
    //@Log(title = "风机信息表修改", action = "111")
    @RequiresPermissions("gen:windturbine:edit")
    @PostMapping("/edit")
    @ResponseBody
    public AjaxResult editSave(Windturbine record)
    {
        return toAjax(windturbineService.updateByPrimaryKeySelective(record));
    }

    
    /**
   	 * 根据主键查询
   	 * 
   	 * @param id
   	 * @param mmap
   	 * @return
   	 */
   	@PostMapping("/get/{id}")
   	@ApiOperation(value = "根据id查询唯一", notes = "根据id查询唯一")
   	public Windturbine edit(@PathVariable("id") String id) {
   		return windturbineService.selectByPrimaryKey(id);
   	}
    

	
}
